function [xapp,yapp,xtest,ytest,xtest1,xtest2]=gdataset(n,nbapp,nbtest,sigma)

% Usage
% [xapp,yapp,xtest,ytest,xtest1,xtest2]=dataset(String,nbapp,nbtest,sigma)
%
%
%  possible dataset creation
%  
%    'Mixture'  : 2 classes         
%          First class is a a mixture of gaussian
%          which takes in sandwiches the second class 
%
%           
%  'Gaussian' : is a classical dataset with two gaussianly distributed
%         classes with similar svModel.tempiances but different means
%
%  'Checkers': Checkers
%
%  'MultiClassChecker'
%          
%  'Clowns' : The Clowns
%
%   'CosExp': sign( 0.95*cos(0.5*(exp(x)-1)))); for x in [0...4]
%       xtest1,xtest2 meshgrid of xtest

xapp=[];
yapp=[];
xtest=[];
ytest=[];
xtest1=[];
xtest2=[];
switch lower(n)
    
case 'mixture'
    
    %------------------Data Learn Generation -------------------------
    if nargin <4
        sigma=0.4;
    end;
    nbapp=round(nbapp/3);
    x1=sigma*randn(1,nbapp)+0.3;
    x2=sigma*randn(1,nbapp)-0.3;
    x3=sigma*randn(1,nbapp)-1;
    y1=sigma*randn(1,nbapp)+0.5;
    y2=sigma*randn(1,nbapp)-0.5;
    y3=sigma*randn(1,nbapp)-1;
    
    xapp=[x1 x2 x3; y1 y2 y3];
    yapp=[ones(1,nbapp) -ones(1,nbapp) ones(1,nbapp)]';
    
    
    %----------------Data Test Generation ---------------------------
    nbtest=round(nbtest/3);
    xt=sigma*randn(1,nbtest)+0.3;
    xt2=sigma*randn(1,nbtest)-0.3;
    xt3=sigma*randn(1,nbtest)-1;
    yt=sigma*randn(1,nbtest)+0.5+0.7*xt;
    yt2=sigma*randn(1,nbtest)-0.5;
    yt3=sigma*randn(1,nbtest)-1-0.7*xt;
    xtest=[xt xt2 xt3; yt yt2 yt3];
    
    
    ytest=[ones(1,nbtest) -ones(1,nbtest) ones(1,nbtest)]';
    return
case 'gaussian'
    nbapp=round(nbapp/2);
    nbtest=round(nbtest/2);
    if nargin <4
        sigma=0.2;
    end;
    x1=sigma*randn(1,nbapp)+0.3;
    x2=sigma*randn(1,nbapp)-0.3;
    y1=sigma*randn(1,nbapp)+0.3;
    y2=sigma*randn(1,nbapp)-0.3;
    xapp=[x1 x2; y1 y2]';
    yapp=[ones(1,nbapp) -ones(1,nbapp)]';
    x1=sigma*randn(1,nbtest)+0.3;
    x2=sigma*randn(1,nbtest)-0.3;
    y1=sigma*randn(1,nbtest)+0.3;
    y2=sigma*randn(1,nbtest)-0.3;
    xtest=[x1 x2; y1 y2]';
    ytest=[ones(1,nbtest) -ones(1,nbtest)]';
    
case 'checkers'
    xapp=[];
    yapp=[];
    nb=floor(nbapp/16);
    for i=-2:1;
        for j=-2:1;
            xapp=[xapp; [i+(1+sigma)*rand(nb,1) j+(1+sigma)*rand(nb,1)]];
            %(2*rem((i+j+4),2)-1)
            yapp=[yapp; (2*rem((i+j+4),2)-1)*ones(nb,1)];
        end;
    end;
    xtest=[];
    ytest=[];
    nb=floor(nbtest/16);
    for i=-2:1;
        for j=-2:1;
            xtest=[xtest; [i+(1+sigma)*rand(nb,1) j+(1+sigma)*rand(nb,1)]];
            %(2*rem((i+j+4),2)-1)
            ytest=[ytest; (2*rem((i+j+4),2)-1)*ones(nb,1)];
        end;
    end;
    
    
case 'clowns'
    nbapp=round(nbapp/2);
    nbtest=round(nbtest/2);
    
    x1 = (6*rand(nbapp,1)-3); 
    x2 = x1.^2 + randn(nbapp,1); 
    x0 = sigma*randn(nbapp,2)+(ones(nbapp,1)*[0 6]); 
    xapp = [x0;[x1 x2]]; 
    xapp = (xapp-ones(2*nbapp,1)*mean(xapp))*diag(1./std(xapp)); 
    yapp = [ones(nbapp,1); -ones(nbapp,1)]; 
    
    if nbtest>0
        x1 = (6*rand(nbtest,1)-3); 
        x2 = x1.^2 + randn(nbtest,1); 
        x0 = sigma*randn(nbtest,2)+(ones(nbtest,1)*[0 6]); 
        xtest = [x0;[x1 x2]]; 
        xtest = (xtest-ones(2*nbtest,1)*mean(xtest))*diag(1./std(xtest)); 
        ytest = [ones(nbtest,1); -ones(nbtest,1)]; 
    end;
    
    

    
case 'cosexp'   
    nbtest=floor(sqrt(nbtest));
    [xi]=rand(nbapp,2);
    yapp= sign(0.95*cos(0.5*(exp(4*xi(:,1))-1))-(2*xi(:,2)-1));
    xapp = [4*xi(:,1) (2*xi(:,2)-1)];
    [xtest1 xtest2]  = meshgrid(linspace(0,4,nbtest),linspace(-1,1,nbtest)); 
    nn = length(xtest1); 
    xtest = [reshape(xtest1 ,nn*nn,1) reshape(xtest2 ,nn*nn,1)]; 
    ytest = sign(0.95*cos(0.5*(exp(4*xtest(:,1))-1))-(2*xtest(:,2)-1));
    
 case 'multiclasscheckers'
     xapp=[];
     yapp=[];
     nb=floor(nbapp/16);
     nb2 = floor(nbtest/16);
    % keyboard
     for i=-2:1;
         for j=-2:1;
             xapp=[xapp; [i+(0.8+sigma)*rand(nb,1) j+(0.8+sigma)*rand(nb,1)]];
             xtest=[xtest; [i+(0.8+sigma)*rand(nb2,1) j+(0.8+sigma)*rand(nb2,1)]];              
             if rem(abs(i),2)==0 && rem(abs(j),2)==0
                 yapp=[yapp; 1*ones(nb,1)];
                 ytest=[ytest; 1*ones(nb2,1)];
             elseif rem(abs(i),2)==1 && rem(abs(j),2)==0
                 yapp=[yapp; 2*ones(nb,1)];
                 ytest=[ytest; 2*ones(nb2,1)];
             elseif rem(abs(i),2)==0 && rem(abs(j),2)==1
                 yapp=[yapp; 3*ones(nb,1)];
                 ytest=[ytest; 3*ones(nb2,1)];
             elseif rem(abs(i),2)==1 && rem(abs(j),2)==1
                 yapp=[yapp; 4*ones(nb,1)]; 
                 ytest=[ytest; 4*ones(nb2,1)]; 
             end;            
         end;
     end;
     
     
 
 case 'multiclassgaussian'     
     
     mean1=[1 1];
     mean2=[-1 1];
     mean3= [0 -1];
     x1=sigma*randn(nbapp,2)+ones(nbapp,1)*mean1;
     y1= ones(nbapp,1);
     x2=sigma*randn(nbapp,2)+ones(nbapp,1)*mean2;
     y2=2*ones(nbapp,1);
     x3=sigma*randn(nbapp,2)+ones(nbapp,1)*mean3;
     y3=3*ones(nbapp,1);
     Ytarget=[1 2 3];
     xapp=[x1;x2;x3];
     yapp=[y1;y2;y3];
     nbapp=size(xapp,1);
    
        
    %----------------Data Test Generation ---------------------------
     x1=sigma*randn(nbtest,2)+ones(nbtest,1)*mean1;
     y1= ones(nbtest,1);
     x2=sigma*randn(nbtest,2)+ones(nbtest,1)*mean2;
     y2=2*ones(nbtest,1);
     x3=sigma*randn(nbtest,2)+ones(nbtest,1)*mean3;
     y3=3*ones(nbtest,1);
     Ytarget=[1 2 3];
     xtest=[x1;x2;x3];
     ytest=[y1;y2;y3];
     nbapp=size(xapp,1);
     
 case 'carpette'
     xapp=2*rand(nbapp,2)-1;
     xtest=2*rand(nbtest,2)-1;
     yapp = carpette(xapp);
     ytest = carpette(xtest); 
     
 case 'oneclass'
     xapp = sigma*randn(nbapp,2);
     xtest =  [(sigma)*randn(floor(nbtest/2),2); 1.15+sigma*randn(ceil(nbtest/2),2)];
     yapp = ones(nbapp,1);
     ytest =  [ones(floor(nbtest/2),1); -ones(ceil(nbtest/2),1)];
     

end;
xapp = xapp';
xtest = xtest'; 


% ======================================
function color = carpette(x)

color = ones(length(x),1);
r2 = x(:,1).*x(:,1) +x(:,2).*x(:,2) ;
r2(r2<0.04) = 10;
c2 = find(r2<0.25);
color(c2(x(c2,1)>=0)) = 2;
r2(c2) = 10;
c2 = find(r2<0.64);
color(c2(x(c2,1)<0)) = 2;
r2(c2) = 10;
c2 = find(r2<10);
color(c2(x(c2,2)<0)) = 3;
color(c2(x(c2,2)>=0)) = 4;

% {
%     double r2 = x*x + y*y;
% 
%     if (r2 < 0.04) return(0);
%     if (r2 < 0.25) return(x < 0.0 ? 0 : 1);
%     if (r2 < 0.64) return(x < 0.0 ? 1 : 0);
%     return(y > 0.0 ? 2 : 3);
% }